/*
 *                  BioJava development code
 *
 * This code may be freely distributed and modified under the
 * terms of the GNU Lesser General Public Licence.  This should
 * be distributed with the code.  If you do not have a copy,
 * see:
 *
 *      http://www.gnu.org/copyleft/lesser.html
 *
 * Copyright for this code is held jointly by the individual
 * authors.  These should be listed in @author doc comments.
 *
 * For more information on the BioJava project and its aims,
 * or to join the biojava-l mailing list, visit the home page
 * at:
 *
 *      http://www.biojava.org/
 * 
 * Created on Feb 24, 2006
 *
 */
package org.synbiowave.biobrick;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.zip.GZIPInputStream;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.XMLReader;

// TODO: Auto-generated Javadoc
/**
 * The Class BioBrickManager.
 */
public class BioBrickManager{

	/**
	 * Instantiates a new bio brick manager.
	 */
	public BioBrickManager(){}
	
	/**
	 * Read bio brick features.
	 * 
	 * @param String biobrick id
	 * 
	 * @return List<Map<String,String>> features
	 * 
	 * @throws IOException Signals that an I/O exception has occurred.
	 * @throws SAXException the SAX exception
	 */
	public static List<Map<String,String>> readBioBrickFeatures(String id) throws IOException,SAXException{
		String url="http://partsregistry.org/das/parts/features/?segment="+id;
		InputStream stream=getBioBrickStream(url);
		XMLReader xmlreader=getBioBrickReader();
		BioBrick_DAS_Feature_Handler cont_handle=new BioBrick_DAS_Feature_Handler();
		xmlreader.setContentHandler(cont_handle);
		xmlreader.setErrorHandler(new org.xml.sax.helpers.DefaultHandler());
		InputSource insource=new InputSource();
		insource.setByteStream(stream);
		xmlreader.parse(insource);
		return cont_handle.getFeatures();}
	
	/**
	 * Read bio brick sequence.
	 * 
	 * @param String biobrick id.
	 * 
	 * @return String sequence.
	 * 
	 * @throws IOException Signals that an I/O exception has occurred.
	 * @throws SAXException the SAX exception
	 */
	public static String readBioBrickSequence(String id) throws IOException,SAXException{
		String url="http://partsregistry.org/das/parts/dna/?segment="+id;
		InputStream stream=getBioBrickStream(url);
		XMLReader xmlreader=getBioBrickReader();
		BioBrick_DAS_DNA_Handler cont_handle=new BioBrick_DAS_DNA_Handler();
		xmlreader.setContentHandler(cont_handle);
		xmlreader.setErrorHandler(new org.xml.sax.helpers.DefaultHandler());
		InputSource insource=new InputSource();
		insource.setByteStream(stream);
		xmlreader.parse(insource);
		return cont_handle.getSequence();}

	/**
	 * Get the complete set of biobrick ids.
	 * 
	 * @return Sorted set of biobrick ids.
	 * 
	 * @throws IOException Signals that an I/O exception has occurred.
	 * @throws SAXException the SAX exception
	 */
	public static SortedSet<String> readBioBrickIDs() throws IOException,SAXException{
		String url="http://partsregistry.org/das/parts/entry_points/";
		InputStream stream=getBioBrickStream(url);
		XMLReader xmlreader=getBioBrickReader();
		BioBrick_DAS_Entry_Points_Handler cont_handle=new BioBrick_DAS_Entry_Points_Handler();
		xmlreader.setContentHandler(cont_handle);
		xmlreader.setErrorHandler(new org.xml.sax.helpers.DefaultHandler());
		InputSource insource=new InputSource();
		insource.setByteStream(stream);
		xmlreader.parse(insource);    
		return cont_handle.getIDs();}
	
	/**
	 * Gets the biobrick stream.
	 * 
	 * @param urlPath the url path.
	 * 
	 * @return the biobrick stream.
	 * 
	 * @throws IOException Signals that an I/O exception has occurred.
	 */
	public static InputStream getBioBrickStream(String urlPath) throws IOException{
		URL url=new URL(urlPath);
		HttpURLConnection huc=(HttpURLConnection)url.openConnection();
		huc.setRequestProperty("Accept-Encoding","gzip");
		String contentEncoding=huc.getContentEncoding();
		InputStream stream=huc.getInputStream();
		if(contentEncoding!=null){
			if(contentEncoding.indexOf("gzip")!=-1){stream=new GZIPInputStream(stream);}}
		return stream;}

	/**
	 * Gets the bio brick reader.
	 * 
	 * @return the biobrick reader.
	 * 
	 * @throws SAXException the SAX exception
	 */
	private static XMLReader getBioBrickReader() throws SAXException{
		SAXParserFactory spfactory=SAXParserFactory.newInstance();
		spfactory.setValidating(false);
		SAXParser saxParser = null ;
		try{saxParser=spfactory.newSAXParser();}
		catch(ParserConfigurationException e){System.out.println(e.toString());}
		String vali=System.getProperty("XMLVALIDATION");
		boolean validation=false ;
		if(vali!=null){if(vali.equals("true")){validation=true;}}
		XMLReader xmlreader=saxParser.getXMLReader();
		try{xmlreader.setFeature("http://xml.org/sax/features/validation",validation);}
		catch(SAXException e){System.out.println(e.toString());}
		try{xmlreader.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd",validation);}
		catch(SAXNotRecognizedException e){System.out.println(e.toString());}
		return xmlreader;}}