package org.synbiowave.biojava;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;

import org.biojava.bio.symbol.AlphabetManager;
import org.biojava.bio.symbol.SimpleSymbolList;
import org.biojava.bio.symbol.SymbolList;
import org.biojavax.Namespace;
import org.biojavax.RichObjectFactory;
import org.biojavax.SimpleRichAnnotation;
import org.biojavax.bio.db.HashRichSequenceDB;
import org.biojavax.bio.db.RichSequenceDB;
import org.biojavax.bio.seq.RichFeature;
import org.biojavax.bio.seq.RichSequence;
import org.biojavax.bio.seq.RichSequenceIterator;
import org.biojavax.bio.seq.SimplePosition;
import org.biojavax.bio.seq.SimpleRichLocation;
import org.biojavax.bio.seq.SimpleRichSequence;
import org.biojavax.bio.seq.RichLocation.Strand;
import org.biojavax.bio.seq.io.GenbankFormat.Terms;
import org.synbiowave.AbstractSynBioWaveRobot;
import org.synbiowave.workspace.Sequence;

import com.google.wave.api.Annotation;
import com.google.wave.api.Wavelet;
import com.google.wave.api.event.Event;

/**
 * Sequence file import/export (based on the BioJava - Project(see biojava.org)).
 * This class contains all methods needed for sequence file import, conversion and export.
 */
public class ImportExport {
	
	/** Default namespace constant for the BioJavaX classes. */
	private static final Namespace ns=RichObjectFactory.getDefaultNamespace();

	/** Basic constructor for the ImportExport class. */
	public ImportExport() {initializeFileFormats();}

	
	/** Method for adding all supported file formats. */
	private void initializeFileFormats() {
		RichSequence.IOTools.registerFormat(org.biojavax.bio.seq.io.FastaFormat.class);
		RichSequence.IOTools.registerFormat(org.biojavax.bio.seq.io.EMBLFormat.class);
		RichSequence.IOTools.registerFormat(org.biojavax.bio.seq.io.EMBLxmlFormat.class);
		RichSequence.IOTools.registerFormat(org.biojavax.bio.seq.io.GenbankFormat.class);
		RichSequence.IOTools.registerFormat(org.biojavax.bio.seq.io.INSDseqFormat.class);
		RichSequence.IOTools.registerFormat(org.biojavax.bio.seq.io.UniProtFormat.class);
		RichSequence.IOTools.registerFormat(org.biojavax.bio.seq.io.UniProtXMLFormat.class);}
	
	/**
	 * Method for the sequence file import.
	 * 
	 * @param The file input stream.
	 * @param The robot itself.
	 * @param The current event.
	 * 
	 * @throws IOException 
	 * 
	 */
	public boolean read(InputStream in,AbstractSynBioWaveRobot servlet,Event event) throws IOException{
		return read(in,servlet,event.getWavelet());}
	
	/**
	 * Method for the sequence file import.
	 * 
	 * @param The file input stream.
	 * @param The robot itself.
	 * @param The current wavelet.
	 * 
	 * @throws IOException 
	 * 
	 */
	public boolean read(InputStream in,AbstractSynBioWaveRobot servlet,Wavelet wavelet) throws IOException{
		Map<String,String> sequences=new HashMap<String,String>();
		RichSequenceIterator rsi=RichSequence.IOTools.readStream(new BufferedInputStream(in),ImportExport.ns);
		RichSequence rs;
		while(rsi.hasNext()){
			try{
				rs=rsi.nextRichSequence();
				Sequence sequence=new Sequence();
				if(rs.getDescription()!=null&&rs.getDescription().length()>1){sequence.setName(rs.getDescription());}
				else if(rs.getAccession()!=null&&rs.getAccession().length()>1){sequence.setName(rs.getAccession());}
				else{sequence.setName(wavelet.getRobotAddress());}
				sequence.setAuthor(wavelet.getRobotAddress());
				if(rs.getAlphabet().getName()!=null){sequence.setType(rs.getAlphabet().getName());}
				sequence.setSequence(rs.seqString());
				for(org.biojava.bio.seq.Feature f:rs.getFeatureSet()){
					sequence.addAnnotation(f.getType(),f.getLocation().getMin(),f.getLocation().getMax());}
				sequence.publish(servlet,wavelet,true);
				sequences.put(sequence.getName(),sequence.getSequence());}
			catch(Exception e){wavelet.reply("\nSorry Captain, can't import the sequence(s).");}}
		if(!sequences.isEmpty()){
			SequenceUtils.showGadgetView(wavelet.reply("\n"),sequences);
			return true;}
		else{return false;}}

	/**
	 * Method for the sequence file export.
	 * 
	 * @param The file output stream.
	 * @param List of sequences to export.
	 * @param Format as String representation (e.q. "fasta")
	 * 
	 */
	public void write(OutputStream out,List<Sequence> sequences,String format){
		try{
			RichSequenceDB rsdb=new HashRichSequenceDB();
			if(!sequences.isEmpty()){
				for(Sequence sequence:sequences){rsdb.addSequence(convertToRichSequence(sequence));}
				if(format.equalsIgnoreCase("fasta")){
					RichSequence.IOTools.writeFasta(out,rsdb.getRichSequenceIterator(),ns);}
				else if(format.equalsIgnoreCase("embl")){
					RichSequence.IOTools.writeEMBL(out,rsdb.getRichSequenceIterator(),ns);}
				else if(format.equalsIgnoreCase("genbank")){
					RichSequence.IOTools.writeGenbank(out,rsdb.getRichSequenceIterator(),ns);}
				out.close();}}
		catch(Exception e){}
		}
	
	/**
	 * Convert to rich sequence.
	 * 
	 * @param org.synbiowave.workspace.Sequence
	 * 
	 * @return org.biojava.bio.seq.RichSequence
	 * 
	 * @throws Exception 
	 * 
	 */
	@SuppressWarnings("unchecked")
	public RichSequence convertToRichSequence(Sequence sequence) throws Exception{
		SymbolList syms=new SimpleSymbolList(AlphabetManager.alphabetForName(sequence.getType()).getTokenization("token"),this.genbankReady(sequence.getSequence()));
		RichSequence rs=new SimpleRichSequence(ns,this.genbankReady(sequence.getName()),this.genbankReady(sequence.getName()),1,syms,new Double(1.0));
		List<Annotation> features=sequence.getAnnotations();
		RichFeature.Template templ;
        for(Annotation feature:features){
        	templ=new RichFeature.Template();
            templ.annotation=new SimpleRichAnnotation();
            templ.sourceTerm=Terms.getGenBankTerm();
            templ.typeTerm=RichObjectFactory.getDefaultOntology().getOrCreateTerm(this.genbankReady(feature.getValue()));
            templ.featureRelationshipSet=new TreeSet();
            templ.rankedCrossRefs=new TreeSet();
            templ.location=new SimpleRichLocation(new SimplePosition(feature.getRange().getStart()),new SimplePosition(feature.getRange().getEnd()),1,Strand.UNKNOWN_STRAND,null);
            rs.createFeature(templ);}
		return rs;}
	
	/**
	 * Prepares a String for the storing in a sequence format.
	 * 
	 * @param String original string.
	 * 
	 * @return String cleaned string.
	 * 
	 */
	private String genbankReady(String raw){
		String gbr=raw.replaceAll("[\\s\\W]","_");
		while(gbr.contains("__")){gbr=gbr.replaceAll("__","_");}
		if(gbr.startsWith("_")){gbr=gbr.substring(1);}
		if(gbr.endsWith("_")){gbr=gbr.substring(0,gbr.length()-1);}
		return gbr;}}