package org.synbiowave.biojava;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.biojava.bio.seq.DNATools;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.RNATools;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.synbiowave.workspace.Sequence;

import com.google.wave.api.Annotation;
import com.google.wave.api.Blip;
import com.google.wave.api.Gadget;
import com.google.wave.api.Range;

// TODO: Auto-generated Javadoc
/**
 * Class with methods for the SynBioWave sequence displays.
 */
public class SequenceUtils {
	
	/** The aminoacid color code. */
	private static char[][] aColorCode={{'Y','F','A','I','L','M','V'},
										 {'W'},	
										 {'G'},	
										 {'P'},	
										 {'S','D','E','T'},
										 {'R','N','K'},
										 {'H'},	
										 {'Q'},	
										 {'C'}};
		
	/** The aminoacid colors. */
	private static String[] aColors={"green","mediumturquoise","magenta","gold","red","blue","cornflowerblue","gray","saddlebrown"};
	
	/** The nucleotide color code. */
	private static char[][] nColorCode={{'U','T'},
										 {'A'},	
										 {'G'},
										 {'C'}};

	/** The nucleotide colors. */
	private static String[] nColors={"red","green","black","blue"};

	/**
	 * Instantiates a new sequence utility.
	 */
	public SequenceUtils(){}
	
	/**
	 * Checks the sequence type.
	 * 
	 * @param String sequence.
	 * 
	 * @return String sequence type.
	 * 
	 */
	public static String determineType(String sequence){
		String type="DNA";
		try{DNATools.createDNA(sequence);}
		catch(IllegalSymbolException e){
			type="RNA";
			try{RNATools.createRNA(sequence);}
			catch(IllegalSymbolException se){
				type="PROTEIN-TERM";
				try{ProteinTools.createProtein(sequence);}
				catch(IllegalSymbolException ise){
					type="null";}}}
		return type;}

	/**
	 * Get the color for a letter of a sequence of specific sequence type.
	 * 
	 * @param char letter of a sequence.
	 * @param String sequence type.
	 * 
	 * @return String color.
	 * 
	 */
	public static String getColor(char letter, String type){
		if(type == null) return "black";
		char[][] colorCode=SequenceUtils.nColorCode;
		String[] colors=SequenceUtils.nColors;
		if(type.contains("PROTEIN")){
			colorCode=SequenceUtils.aColorCode;
			colors=SequenceUtils.aColors;}
		for(int r=0;r<colorCode.length;r++){
			for(int c=0;c<colorCode[r].length;c++){
				if(colorCode[r][c]==Character.toUpperCase(letter)){return colors[r];}}}
		return "black";}
	
	/**
	 * Gets the complete scaling in parts of length 50.
	 * 
	 * @param int sequence length.
	 * 
	 * @return ArrayList<String> of complete scaling rows.
	 * 
	 */
	public static ArrayList<String> getScale(int length){
		ArrayList<String> scalings=new ArrayList<String>();
		StringBuffer scale=new StringBuffer();
		StringBuffer numbers=new StringBuffer();
		int wait=0;
		for(int i=1;i<length+1;i++){
			if((i%5)==0){
				scale.append('|');
				numbers.append(i);
				wait=String.valueOf(i).length()-1;}
			else{
				scale.append('\'');
				if(wait==0){numbers.append(' ');}
				else{wait--;}}
			if((i%60)==0){
				scalings.add(scale.append('\n').append(numbers.append('\n')).toString());
				scale=new StringBuffer();
				numbers=new StringBuffer();
				wait=0;}}
		scalings.add(scale.append('\n').append(numbers.append('\n')).toString());
		return scalings;}
	
	/**
	 * Get a HTML view of the sequence.
	 * 
	 * @param String sequence.
	 * 
	 * @return String complete HTML view.
	 * 
	 */
	public static String getHTMLView(String sequence){
		String type=SequenceUtils.determineType(sequence);
		StringBuffer scale=new StringBuffer();
		StringBuffer numbers=new StringBuffer();
		int wait=0;
		for(int i=1;i<sequence.length()+1;i++){
			if((i%5)==0){
				scale.append('|');
				numbers.append(i);
				wait=String.valueOf(i).length()-1;}
			else{
				scale.append('\'');
				if(wait==0){numbers.append("&nbsp;");}
				else{wait--;}}}
		String htmlView="(br /)"+scale+"(br /)"+numbers+"(br /)";
		char[][] colorCode=SequenceUtils.nColorCode;
		String[] colors=SequenceUtils.nColors;
		if(type.contains("PROTEIN")){
			colorCode=SequenceUtils.aColorCode;
			colors=SequenceUtils.aColors;}
		char[] sa=sequence.toUpperCase().toCharArray();
		for(char sc:sa){
			for(int r=0;r<colorCode.length;r++){
				for(int c=0;c<colorCode[r].length;c++){
					if(colorCode[r][c]==sc){
						htmlView += "(span style='color:"+colors[r]+";')"+sc+"(/span)";}}}}
		return htmlView+"(br /)(br /)";}
	
	/**
	 * Get the HTML view.
	 * 
	 * @param List<String> of sequences.
	 * 
	 * @return String complete HTML view.
	 * 
	 */
	public static String getHTMLView(List<Sequence> sequences){
		int maxLength=0;
		for(Sequence sequence:sequences)
		{
			if(sequence.getSequence().length()>maxLength)
			{
				maxLength = sequence.getSequence().length();
			}
		}
		
		StringBuffer scale=new StringBuffer();
		StringBuffer numbers=new StringBuffer();
		
		int wait=0;
		for(int i=1;i<maxLength+1;i++){
			if((i%5)==0){
				scale.append('|');
				numbers.append(i);
				wait=String.valueOf(i).length()-1;}
			else{
				scale.append('\'');
				if(wait==0){numbers.append("&nbsp;");}
				else{wait--;}}}
		StringBuilder htmlView = new StringBuilder("(br /)"+scale+"(br /)"+numbers+"(br /)");
		for(Sequence sequence:sequences)
		{
			for(char sc:sequence.getSequence().toUpperCase().toCharArray()){
				
				htmlView.append("(span style='color:"+SequenceUtils.getColor(sc,sequence.getType())+";')"+sc+"(/span)");
			}
			htmlView.append("(br /)");
		}
		return htmlView.toString()+"(br /)";}
	
	
	/**
	 * Add a sequence scaling to a specific blip range. 
	 * 
	 * @param Blip the specific blip
	 * @param Range the range of the sequence
	 * 
	 */
	public static void addScaling(Blip blip,Range range){
		int index=range.getStart();
		for(String scale:SequenceUtils.getScale(range.getEnd()-range.getStart())){
			blip.at(index).insert("\n\n");
			index+=2;
			blip.at(index).insert(scale);
			blip.range(index,index+scale.length()).annotate("scale","default");
			index+=scale.length();
			index+=scale.indexOf("\n");}
		if(index<blip.length()){blip.at(index).insert("\n\n");}
		blip.range(range.getStart(),index).annotate(Annotation.FONT_FAMILY,"monospace");
		blip.range(range.getStart(),index).annotate("sequence","default");}
	
	/**
	 * Add a color code for the aminoacids or nucleotides of a scaled 
	 * sequence representation. 
	 * 
	 * @param Blip containing the scaled sequence
	 * 
	 */
	public static void addSequenceColors(Blip blip,String type){
		for(Annotation sequence:blip.getAnnotations().get("sequence")){
			for(int index=sequence.getRange().getStart();index<sequence.getRange().getEnd();index++){
				if(Character.isLetter(blip.getContent().charAt(index))){
					blip.at(index).annotate(Annotation.COLOR,SequenceUtils.getColor(blip.getContent().charAt(index),type));}}}}
	
	/**
	 * Display gadget view for a Map of Sequences (old)
	 * 
	 * @param Blip to show in.
	 * @param Map<String,String> of sequence name and sequence pairs
	 * @deprecated replaced by {@link #showGadgetView(Blip blip, List<Sequence> sequences)}
	 */
	public static void showGadgetView(Blip blip,Map<String,String> sequences){
		// Create Sequence list
		List<Sequence> dummyList = new ArrayList<Sequence>();
		for(String key : sequences.keySet())
		{
			dummyList.add(new Sequence(key, sequences.get(key)));
		}
		
		String htmlViewSequences=SequenceUtils.getHTMLView(dummyList);
		String names="(br /)(br /)(br /)";
		for(String name:sequences.keySet()){names+=name + "(br /)";}
		Gadget seqView=new Gadget("http://samnode.de/SequenceView.xml");
		seqView.setProperty("names",names+"(br /)(br /)");
		seqView.setProperty("sequences",htmlViewSequences);
		blip.append(seqView);}
	
	/**
	 * Display gadget view for a List of Sequences (new)
	 * 
	 * @param Blip to show in.
	 * @param List<Sequence> of sequences to display
	 * 
	 */	
	public static void showGadgetView(Blip blip, List<Sequence> sequences){
		String htmlViewSequences = SequenceUtils.getHTMLView(sequences);
		StringBuilder names = new StringBuilder("(br /)(br /)(br /)(br /)(br /)");
		for(Sequence seq:sequences)
		{
			names.append(seq.getName() + "(br /)");
		}
		Gadget seqView = new Gadget("http://synbiowave-beta.appspot.com/SequenceView.xml");
		seqView.setProperty("names",names+"(br /)(br /)");
		seqView.setProperty("sequences",htmlViewSequences);
		blip.append(seqView);}

	/**
	 * Display gadget view.
	 * 
	 * @param Blip to show in.
	 * @param String sequence name.
	 * @param String sequence.
	 * 
	 * @deprecated replaced by {@link #showGadgetView(Blip blip, List<Sequence> sequences)}
	 * 
	 */
	
	public static void showGadgetView(Blip blip,String name,String sequence){
		Map<String,String> sequences=new HashMap<String,String>();
		sequences.put(name,sequence);
		showGadgetView(blip,sequences);}

	// The character used for seperation in the serialized Sequence

	private static final char seperator = '#';
	
	/*
	 * Serializes a Sequence for storage in Gadget state as a String
	 * Order is name, author, type, sequence
	 * TODO: other properties
	 */

	public static String Serialize(Sequence seq)
	{
		String output = "";
		output += seq.getName();
		output += seperator;
		output += seq.getAuthor();
		output += seperator;
		if(seq.getType() != null)
		{
			output += seq.getType();
		}else
		{
			output += "OTHER";
		}
		output += seperator;
		output += seq.getSequence();
		output += seperator;
		
		return output;
	}
	
	/*
	 * Deserializes a Sequence
	 */
	public static Sequence Deserialize(String input)
	{
		String[] parts = input.split(Character.toString(seperator));
		Sequence output = new Sequence();
		try{
			output.setName(parts[0]);
			output.setAuthor(parts[1]);
			output.setType(parts[2]);
			output.setSequence(parts[3]);
		}catch(Exception e)
		{
			
		}
		return output;
	}
}