package org.synbiowave.workspace;

import java.io.IOException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import org.synbiowave.AbstractSynBioWaveRobot;
import org.synbiowave.biojava.SequenceUtils;
import org.waveprotocol.wave.model.id.WaveId;
import org.waveprotocol.wave.model.id.WaveletId;

import com.google.wave.api.Annotation;
import com.google.wave.api.Blip;
import com.google.wave.api.Element;
import com.google.wave.api.ElementType;
import com.google.wave.api.JsonRpcResponse;
import com.google.wave.api.Range;
import com.google.wave.api.Wavelet;
import com.google.wave.api.JsonRpcConstant.ParamsProperty;

/**
 * This class provides all methods for the creation, the temporal storing and the display of sequences.
 * This class also represents a useful sequence format to work on and update sequences in the workspace 
 * wave.
 * The static methods Workspace.getAllSequences(AbstractSynBioWaveRobot servlet,Wavelet wavelet) 
 * and Workspace.getActiveSequences(AbstractSynBioWaveRobot servlet,Wavelet wavelet) return lists of 
 * sequences of this format.
 * After altering this sequences, the can be updated simply by using there 
 * publish(AbstractSynBioWaveRobot servlet,Wavelet wavelet) method.
 */
public class Sequence {
	
	/** The blip ID of this sequence in the workspace wave. */
	private String blipId=null;
	/** The name of the sequence. */
	private String name="default";
	/** The author of the sequence. */
	private String author="default";
	/** The sequence type (DNA,RNA,PROTEIN_TERM,OTHER) */
	private String type="OTHER";
	/** The sequence. */
	private String sequence="";
	/** List of the sequence annotations */
	private List<Annotation> annotations;
	
	/** 
	 * Basic constructor
	 */
	public Sequence(){this.annotations=new ArrayList<Annotation>();}
	
	/** 
	 * Extended constructor
	 */
	public Sequence(String name,String sequence){
		this.name=name;
		this.sequence=sequence;
		this.type=SequenceUtils.determineType(sequence);
		this.annotations=new ArrayList<Annotation>();
	}
	
	/** 
	 * Constructor for the the workspace translation. 
	 * 
	 * @param Blip sequence container
	 * @param Wavelet wavelet
	 * 
	 */
	public Sequence(Blip container){
		String[] sequence=container.getContent().split("\n");
		this.blipId=container.getBlipId();
		this.name=sequence[3];
		this.author=container.getAnnotations().get("author").get(0).getValue();
		this.type=container.getAnnotations().get("type").get(0).getValue();
		this.sequence=sequence[5];
		this.annotations=new ArrayList<Annotation>();
		for(Annotation annotation:container.getAnnotations()){if(annotation.getName().contains("annotation")){this.annotations.add(annotation);}}}
	
	/**
	 * Set the sequence name.
	 * 
	 * @param String name.
	 * 
	 */
	public void setName(String name){this.name=this.jsonReady(name);}
	
	/**
	 * Set the author of the sequence.
	 * 
	 * @param String author.
	 * 
	 */
	public void setAuthor(String author){this.author=this.jsonReady(author);}
	
	/**
	 * Set the sequence type (DNA,RNA,PROTEIN-TERM).
	 * 
	 * @param String type.
	 * 
	 */
	public void setType(String type){this.type=this.jsonReady(type);}
	
	/**
	 * Set the sequence name.
	 * 
	 * @param String name.
	 * 
	 */
	public void setSequence(String sequence){this.sequence=this.jsonReady(sequence.replaceAll("\\s",""));}
	
	/**
	 * Add a new sequence annotation/feature.
	 * 
	 * @param String name.
	 * @param int start.
	 * @param int end.
	 * 
	 */
	public void addAnnotation(String name,int start,int end){this.annotations.add(new Annotation("annotation"+this.annotations.size(),this.jsonReady(name),start,end));}
	
	/**
	 * Get the sequence name.
	 * 
	 * @return String name.
	 * 
	 */
	public String getName(){return name;}

	/**
	 * Get the author of the sequence.
	 * 
	 * @return String author.
	 * 
	 */
	public String getAuthor(){return author;}

	/**
	 * Get the sequence type (DNA,RNA,PROTEIN-TERM).
	 * 
	 * @return String type.
	 * 
	 */
	public String getType(){return type;}

	/**
	 * Get the sequence.
	 * 
	 * @return String sequence.
	 * 
	 */
	public String getSequence(){return sequence;}
	
	/**
	 * Get the sequence annotations/features.
	 * 
	 * @return List<Annotation> of annotations.
	 * 
	 */
	public List<Annotation> getAnnotations(){return this.annotations;}
	
	/**
	 * Get the containers blipId in the workspace.
	 * 
	 * @return String blipId.
	 * 
	 */
	public String getBlipId(){return blipId;}
	
	/**
	 * Add/Update the sequence in the workspace and show it on the wave.
	 * 
	 * @param AbstractSynBioWaveRobot servlet.
	 * @param Wavelet wavelet.
	 * @param boolean active state of the sequence.
	 * 
	 */
	public void publish(AbstractSynBioWaveRobot servlet,Wavelet wavelet,boolean active){
		try{
			String active_flag="";
			if(active){active_flag="true";}
			String[] wavemarks=wavelet.getDataDocuments().get("workspace").split("\n");
			Wavelet workspace=servlet.blindWavelet(new WaveId(wavemarks[0],wavemarks[1]),new WaveletId(wavemarks[0],wavemarks[2]));
			Blip container;
			if(blipId!=null){
				container=workspace.getBlips().get(blipId);
				container.all().delete();}
			else{container=workspace.reply("\n");}
			container.append("\n");
			container.append(new Element(ElementType.LABEL,createProperties("check_lab","Active:")));
			container.append(new Element(ElementType.CHECK,createProperties("check_elem",active_flag)));
			container.append("\n"+this.name.replaceAll("[\n\r]"," ")+"\n");
			DateFormat df=DateFormat.getDateTimeInstance(DateFormat.FULL,DateFormat.MEDIUM );
			int index=container.length();
			container.append("// "+df.format(new Date()));
			container.range(index,container.length()).annotate(Annotation.COLOR,"red");
			container.append("\n");
			index=container.length();
			container.append(this.sequence+"\n");
			container.range(index,container.length()).annotate(Annotation.COLOR,"white");
			container.range(index,container.length()).annotate("author",this.author);
			if(this.type!=null){container.range(index,container.length()).annotate("type",this.type);}
			else{container.range(index,container.length()).annotate("type",SequenceUtils.determineType(this.sequence));}
			for(int i=0;i<this.annotations.size();i++){
				if(this.annotations.get(i).getRange().getStart()>0&&this.annotations.get(i).getRange().getEnd()>0&&this.annotations.get(i).getRange().getStart()<=this.sequence.length()&&this.annotations.get(i).getRange().getEnd()<=this.sequence.length()){
					container.range(this.annotations.get(i).getRange().getStart(),this.annotations.get(i).getRange().getEnd()).annotate(this.annotations.get(i).getName(),this.annotations.get(i).getValue());}
				else{
					container.range(1,this.sequence.length()).annotate(this.annotations.get(i).getName(),this.annotations.get(i).getValue());}}
			List<JsonRpcResponse> response=servlet.submit(workspace,wavemarks[3]);
			this.blipId=(String)response.get(0).getData().get(ParamsProperty.NEW_BLIP_ID);}
		catch(IOException e){wavelet.reply("\nSorry Captain, but we are experiencing some slight turbulence while connecting to the workspace.");}}

	/**
	 * Display blip view.
	 * 
	 * @param Blip to show in.
	 * 
	 */
	public void showBlipView(Blip blip){
		blip.append(this.sequence);
		SequenceUtils.addScaling(blip,new Range(1,this.sequence.length()));
		if(this.type!=null){SequenceUtils.addSequenceColors(blip,this.type);}
		else{SequenceUtils.addSequenceColors(blip,SequenceUtils.determineType(this.sequence));}}
	
	/**
	 * Display gadget view.
	 * 
	 * @param Blip to show in.
	 * 
	 */
	public void showGadgetView(Blip blip){SequenceUtils.showGadgetView(blip,this.name,this.sequence);}
	
	/**
	 * Create a wave elements property.
	 * 
	 * @param String name.
	 * @param String value.
	 * 
	 */
	private static HashMap<String,String> createProperties(String name,String value) {
		HashMap<String,String> properties=new HashMap<String,String>();
		properties.put("name",name);
		properties.put("value",value);
		return properties;}
	
	/**
	 * Replace json threading characters.
	 * 
	 * @param String raw.
	 * @param String jsonReady.
	 * 
	 */
	private String jsonReady(String raw){return raw.replaceAll("<","(").replaceAll(">",")");}}