package org.synbiowave.workspace;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.synbiowave.AbstractSynBioWaveRobot;
import org.waveprotocol.wave.model.id.WaveId;
import org.waveprotocol.wave.model.id.WaveletId;

import com.google.appengine.repackaged.com.google.common.collect.ImmutableMap;
import com.google.wave.api.Blip;
import com.google.wave.api.BlipContentRefs;
import com.google.wave.api.ElementType;
import com.google.wave.api.FormElement;
import com.google.wave.api.Wavelet;

/**
 * This class provides all methods for retrieving sequences of the workspace or to modify the active state
 * of this sequences.
 */
public class Workspace {
	
	/**
	 * Get all sequences contained in the workspace.
	 * 
	 * @param AbstractSynBioWaveRobot The robot itself.
	 * @param Wavelet the current wavelet.
	 * 
	 * @return List<Sequence> of the sequences retrieved.
	 * 
	 */
	public static List<Sequence> getAllSequences(AbstractSynBioWaveRobot servlet,Wavelet wavelet){
		return getSequences(servlet,wavelet,true);}
	
	/**
	 * Get all sequences contained in the workspace with active state.
	 * 
	 * @param AbstractSynBioWaveRobot The robot itself.
	 * @param Wavelet the current wavelet.
	 * 
	 * @return List<Sequence> of the active sequences.
	 * 
	 */
	public static List<Sequence> getActiveSequences(AbstractSynBioWaveRobot servlet,Wavelet wavelet){
		return getSequences(servlet,wavelet,false);}
	
	/**
	 * The primitive method used from the public methods getAllSequences and getActiveSequences.
	 * 
	 * @param AbstractSynBioWaveRobot The robot itself.
	 * @param Wavelet the current wavelet.
	 * @param Boolean all or only active sequences.
	 * 
	 * @return List<Sequence> of the sequences retrieved.
	 * 
	 */
	private static List<Sequence> getSequences(AbstractSynBioWaveRobot servlet,Wavelet wavelet,boolean all) {
		try{
			List<Sequence> sequences=new ArrayList<Sequence>();
			String[] wavemarks=wavelet.getDataDocuments().get("workspace").split("\n");
			Wavelet workspace=servlet.fetchWavelet(new WaveId(wavemarks[0],wavemarks[1]),new WaveletId(wavemarks[0],wavemarks[2]),wavemarks[3]);
			for(Blip container:workspace.getBlips().values()){
				if(!container.isRoot()){
					BlipContentRefs checkbox=container.first(ElementType.CHECK,FormElement.restrictByName("check_elem"));
					if(all||(checkbox.value()!=null&&checkbox.value().asElement().getProperty("value").equalsIgnoreCase("true"))){
						sequences.add(new Sequence(container));}}}
			return sequences;}
		catch(IOException e){wavelet.reply("\nError while fetching the sequence:\n"+e.toString()+"\nEverything's shiny, Cap'n. Not to fret!\nPlease try again in a moment ...");return null;}}
	
	/**
	 * Set the active flag for a specific workspace sequence.
	 * 
	 * @param AbstractSynBioWaveRobot The robot itself.
	 * @param Wavelet the current wavelet.
	 * @param String blipId of the sequence to set active.
	 * 
	 */
	public static void setActive(AbstractSynBioWaveRobot servlet,Wavelet wavelet,String blipId) {
		try{
			String[] wavemarks=wavelet.getDataDocuments().get("workspace").split("\n");
			Wavelet workspace=servlet.fetchWavelet(new WaveId(wavemarks[0],wavemarks[1]),new WaveletId(wavemarks[0],wavemarks[2]),wavemarks[3]);
			Blip container=workspace.getBlips().get(blipId);
			BlipContentRefs checkbox=container.first(ElementType.CHECK,FormElement.restrictByName("check_elem"));
			if(checkbox.value()!=null){checkbox.updateElement(ImmutableMap.of("value","true"));}
			servlet.submit(workspace,wavemarks[3]);}
		catch(IOException e){wavelet.reply("\nError while setting the active state of the sequence:\n"+e.toString()+"\nEverything's shiny, Cap'n. Not to fret!\nPlease try again in a moment ...");}}
	
	/**
	 * Set the active flag for a specific set of workspace sequences.
	 * 
	 * @param AbstractSynBioWaveRobot The robot itself.
	 * @param Wavelet the current wavelet.
	 * @param List<String> blipIds of the sequences to set active.
	 * 
	 */
	public static void setActive(AbstractSynBioWaveRobot servlet,Wavelet wavelet,List<String> blipIds) {
		try{
			String[] wavemarks=wavelet.getDataDocuments().get("workspace").split("\n");
			Wavelet workspace=servlet.fetchWavelet(new WaveId(wavemarks[0],wavemarks[1]),new WaveletId(wavemarks[0],wavemarks[2]),wavemarks[3]);
			for(String blipId:blipIds){
				Blip container=workspace.getBlips().get(blipId);
				BlipContentRefs checkbox=container.first(ElementType.CHECK,FormElement.restrictByName("check_elem"));
				if(checkbox.value()!=null){checkbox.updateElement(ImmutableMap.of("value","true"));}}
			servlet.submit(workspace,wavemarks[3]);}
		catch(IOException e){wavelet.reply("\nError while setting the active state of the sequences:\n"+e.toString()+"\nEverything's shiny, Cap'n. Not to fret!\nPlease try again in a moment ...");}}
	
	/**
	 * Reset the active state of all workspace sequences.
	 * 
	 * @param AbstractSynBioWaveRobot The robot itself.
	 * @param Wavelet the current wavelet.
	 * 
	 */
	public static void resetAllActivations(AbstractSynBioWaveRobot servlet,Wavelet wavelet) {
		try{
			String[] wavemarks=wavelet.getDataDocuments().get("workspace").split("\n");
			Wavelet workspace=servlet.fetchWavelet(new WaveId(wavemarks[0],wavemarks[1]),new WaveletId(wavemarks[0],wavemarks[2]),wavemarks[3]);
			for(Blip container:workspace.getBlips().values()){
				BlipContentRefs checkbox=container.first(ElementType.CHECK,FormElement.restrictByName("check_elem"));
				if((checkbox.value()!=null&&checkbox.value().asElement().getProperty("value").equalsIgnoreCase("true"))){checkbox.updateElement(ImmutableMap.of("value",""));}}
			servlet.submit(workspace,wavemarks[3]);}
		catch(IOException e){wavelet.reply("\nError while reseting the sequence workspace:\n"+e.toString()+"\nEverything's shiny, Cap'n. Not to fret!\nPlease try again in a moment ...");}}}